<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
		version="2.0"
		xmlns:t="http://kosek.cz/cz.kosek.saxon.gettext.Gettext"
		xmlns:saxon="http://saxon.sf.net/"
		xmlns:xq="java:Xq2xml"
		exclude-result-prefixes="saxon xq">

<xsl:import href="xq2xq.xsl"/>

<xsl:output method="text" encoding="utf-8"/>

<xsl:strip-space elements="*"/>
<xsl:preserve-space elements="xsl:*"/>

<xsl:template match="node()" mode="collect">
  <xsl:copy>
    <xsl:copy-of select="@*"/>
    <xsl:apply-templates mode="collect"/>
  </xsl:copy>
</xsl:template>

<xsl:template match="xsl:import | xsl:include" mode="collect">
  <xsl:copy>
    <xsl:copy-of select="@*"/>
    <xsl:apply-templates mode="collect"/>
  </xsl:copy>
  <xsl:message>Processing <xsl:value-of select="@href"/>...</xsl:message>
  <xsl:apply-templates select="document(@href, .)" mode="collect"/>
</xsl:template>

<xsl:template match="FunctionCall[FunctionQName/data=('t:_', 't:gettext')]" mode="expr">
  <msgid><xsl:value-of select="translate(PathExpr/StringLiteral/data, &quot;'&quot;, '')"/></msgid>
</xsl:template>

<xsl:template match="FunctionCall[FunctionQName/data='t:ngettext']" mode="expr">
  <msgid plural="{translate(PathExpr[2]/StringLiteral/data, &quot;'&quot;, '')}">
    <xsl:value-of select="translate(PathExpr[1]/StringLiteral/data, &quot;'&quot;, '')"/>
  </msgid>
</xsl:template>

<xsl:template match="text()" mode="expr"/>

<xsl:template match="/">
  <xsl:variable name="doc">
    <xsl:apply-templates select="." mode="collect"/>
  </xsl:variable>
  
  <xsl:variable name="gettext">
    <xsl:for-each select="$doc//t:_ | $doc//t:gettext">
      <msgid><xsl:value-of select="."/></msgid>
    </xsl:for-each>
    <xsl:for-each select="$doc//*/@select">
      <xsl:apply-templates select="saxon:parse(xq:convert(string(.)))/*" mode="expr"/>
    </xsl:for-each>
  </xsl:variable>

<xsl:text>msgid ""
msgstr ""
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=UTF-8\n"
"Content-Transfer-Encoding: 8bit\n"
</xsl:text>

  <xsl:for-each-group select="$gettext/msgid[. != '']" group-by=".">
    <xsl:text>&#10;</xsl:text>
    <xsl:text>msgid "</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>"</xsl:text>
    <xsl:text>&#10;</xsl:text>
    <xsl:choose>
      <xsl:when test="@plural">
	<xsl:text>msgid_plural "</xsl:text>
	<xsl:value-of select="@plural"/>
	<xsl:text>"</xsl:text>
	<xsl:text>&#10;</xsl:text>
	<xsl:text>msgstr[0] ""</xsl:text>
	<xsl:text>&#10;</xsl:text>
	<xsl:text>msgstr[1] ""</xsl:text>
	<xsl:text>&#10;</xsl:text>
      </xsl:when>
      <xsl:otherwise>
	<xsl:text>msgstr ""</xsl:text>
	<xsl:text>&#10;</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:for-each-group>
</xsl:template>

</xsl:stylesheet>