package cz.kosek.saxon.gettext;

import java.util.ArrayList;
import java.util.Iterator;
import net.sf.saxon.expr.Expression;
import net.sf.saxon.expr.ExpressionTool;
import net.sf.saxon.expr.SimpleExpression;
import net.sf.saxon.expr.StaticProperty;
import net.sf.saxon.expr.XPathContext;
import net.sf.saxon.instruct.Executable;
import net.sf.saxon.om.Axis;
import net.sf.saxon.om.Item;
import net.sf.saxon.style.ExtensionInstruction;
import net.sf.saxon.trans.XPathException;
import net.sf.saxon.value.TextFragmentValue;

public class GetTextElement extends ExtensionInstruction
{
  public void prepareAttributes() throws XPathException
  {
  }

  public void validate() throws XPathException 
  {
    checkWithinTemplate();
  }

  public boolean mayContainSequenceConstructor()
  {
    return true;
  }
  
  public Expression compile(Executable exec) throws XPathException 
  {
    Expression text = compileSequenceConstructor(exec, iterateAxis(Axis.CHILD), true);
    GetTextInstruction inst = new GetTextInstruction(text);
    //ExpressionTool.makeParentReferences(inst);
    return inst;
  }

  private static class GetTextInstruction extends SimpleExpression
  {
    private Expression text;
    
    public GetTextInstruction(Expression text)
    {
      this.text = text;
      adoptChildExpression(text);
    }
    
    public int getImplementationMethod() 
    {
      return Expression.EVALUATE_METHOD;
    }

    public int computeCardinality() 
    {
      return StaticProperty.ALLOWS_ONE;
    }

    public final boolean createsNewNodes()
    {
        return true;
    }

    public String getExpressionType()
    {
      return "gettext:gettext";
    }
  
    public Iterator iterateSubExpressions()
    {
      ArrayList list = new ArrayList(1);
      if (text != null)
      {
        list.add(text);
      }
      return list.iterator();
    }

    public Item evaluateItem(XPathContext context) throws XPathException 
    {
      return new TextFragmentValue(Gettext.gettext(text.evaluateAsString(context).toString()), "");
    }
  }

  
}
