package cz.kosek.saxon.gettext;

import net.sf.saxon.expr.Expression;
import net.sf.saxon.expr.SimpleExpression;
import net.sf.saxon.expr.StaticProperty;
import net.sf.saxon.expr.XPathContext;
import net.sf.saxon.instruct.Executable;
import net.sf.saxon.om.Item;
import net.sf.saxon.style.ExtensionInstruction;
import net.sf.saxon.trans.XPathException;

public class InitElement extends ExtensionInstruction
{

  Expression locale;
  Expression domain;
  
  public void prepareAttributes() throws XPathException
  {
    String localeAttr = attributeList.getValue("", "locale");
    if (localeAttr == null) 
    {
        localeAttr = "";
    }
    locale = makeAttributeValueTemplate(localeAttr);

    String domainAttr = attributeList.getValue("", "domain");
    if (domainAttr == null) 
    {
      reportAbsence("domain");
      domainAttr = "";
    }
    domain = makeAttributeValueTemplate(domainAttr);
  }

  public void validate() throws XPathException 
  {
      super.validate();
      locale = typeCheck("locale", locale);
      domain = typeCheck("domain", domain);
  }

  public Expression compile(Executable exec) throws XPathException 
  {
    return new InitInstruction(locale, domain);
  }

  private static class InitInstruction extends SimpleExpression
  {
    public static final int LOCALE = 0;
    public static final int DOMAIN = 1;

    public InitInstruction(Expression locale,
                           Expression domain) 
    {
      Expression[] subs = { locale, domain };
      setArguments(subs);
    };

    public int getImplementationMethod() 
    {
      return Expression.EVALUATE_METHOD;
    }

    public int computeCardinality() 
    {
      return StaticProperty.ALLOWS_ZERO;
    }

    public String getExpressionType()
    {
      return "gettext:init";
    }

    public Item evaluateItem(XPathContext context) throws XPathException 
    {
      Gettext.setLocale(arguments[LOCALE].evaluateAsString(context).toString());
      Gettext.setDomain(arguments[DOMAIN].evaluateAsString(context).toString());
      return null;
    }
  }
}
